import io
import json
from contextlib import redirect_stderr, redirect_stdout
from pathlib import Path
from typing import Any, Optional, Sequence

import typer
from click import ClickException
from snowflake.cli._app.__main__ import main


def run_snow_command(
    cmd_args: Sequence[str],
    capture_output: bool = True,
    check: bool = True,
    debug: bool = False,
) -> Any:
    """Run a snow CLI command with proper UV wrapping."""
    if debug:
        cmd_args = "snow" + " ".join(cmd_args)
        print("Running command:", cmd_args)
    stdout = io.StringIO()
    stderr = io.StringIO()
    try:
        if capture_output:
            with redirect_stdout(stdout), redirect_stderr(stderr):
                main(cmd_args)
        else:
            main(cmd_args)
    except SystemExit as e:
        if e.code != 0 and check:
            raise ValueError(stdout.getvalue() + "\n" + stderr.getvalue()) from e
    return stdout.getvalue() + "\n" + stderr.getvalue()


def deploy_streamlit(
    *,
    name: str,
    database: str,
    schema: str,
    role: str,
    title: Optional[str] = None,
    replace: bool = False,
    open_browser: bool = False,
    connection: Optional[str] = None,
    query_warehouse: Optional[str] = None,
    app_warehouse: Optional[str] = None,
    verbose: bool = False,
    debug: bool = False,
    extra_args: Sequence[str] | None = None,
) -> Any:
    """Deploy a Streamlit application using the snow CLI."""
    cmd = ["streamlit", "deploy"]

    if replace:
        cmd.append("--replace")

    if open_browser:
        cmd.append("--open")

    if connection:
        cmd.extend(["--connection", connection])

    # Add environment variables
    cmd.append(f"--env=name={name}")
    if title:
        cmd.append(f"--env=title={title}")

    # Add database, schema, and role
    cmd.extend(["--database", database])
    cmd.extend(["--schema", schema])
    cmd.extend(["--role", role])

    # Handle warehouse overrides through env variables
    if query_warehouse:
        cmd.append(f"--env=query_warehouse={query_warehouse}")
    if app_warehouse:
        cmd.append(f"--env=app_warehouse={app_warehouse}")

    cmd.append(f"--env=database={database}")
    cmd.append(f"--env=schema={schema}")
    cmd.append(f"--env=role={role}")

    if verbose:
        cmd.append("--verbose")

    if debug:
        cmd.append("--debug")

    if extra_args:
        cmd.extend(extra_args)

    try:
        run_snow_command(cmd, capture_output=not verbose, check=True, debug=debug)
    except Exception as e:
        raise e


def drop_streamlit(
    app_name: str,
    database: str,
    schema: str,
    connection: Optional[str] = None,
    verbose: bool = False,
) -> Any:
    """Drop a Streamlit application using the snow CLI."""
    full_app_name = f"{database}.{schema}.{app_name}"
    cmd = ["object", "drop", "streamlit", full_app_name]

    if connection:
        cmd.extend(["--connection", connection])

    return run_snow_command(cmd, capture_output=not verbose)


def drop_spcs_services(
    app_name: str,
    connection: Optional[str] = None,
    verbose: bool = False,
) -> Any:
    """Drop SPCS services associated with a Streamlit app."""
    cmd = [
        "sql",
        "-q",
        f"CALL SYSTEM\\$DELETE_RUNNING_STREAMLIT_INSTANCES('{app_name}')",
    ]

    if connection:
        cmd.extend(["--connection", connection])

    return run_snow_command(cmd, capture_output=not verbose, check=False)


def list_streamlit_apps(
    filter_text: Optional[str] = None,
    connection: Optional[str] = None,
    verbose: bool = False,
) -> list[dict[str, Any]]:
    """List Streamlit applications in Snowflake."""
    cmd = ["object", "list", "streamlit", "--format=json"]

    if filter_text:
        cmd.extend(["--like", f"%{filter_text}%"])

    if connection:
        cmd.extend(["--connection", connection])

    if verbose:
        cmd.append("--verbose")

    result = run_snow_command(cmd, capture_output=True)
    return json.loads(result.stdout)


def init_app(
    output_path: Path,
    template_dir: Path,
    template_name: str,
    variables: dict[str, str],
    verbose: bool = False,
    debug: bool = False,
    force: bool = False,
) -> Any:
    """Initialize a new Streamlit application using the snow CLI."""
    # Remove the directory if it exists and force is True
    if output_path.exists():
        if force:
            import shutil

            shutil.rmtree(output_path)
        else:
            raise ClickException(
                f"The directory {output_path} already exists. Please specify a different path for the project."
            )

    # Ensure we have all required variables
    required_vars = {
        "name": output_path.name,  # Use directory name as app name if not provided
        "database": variables.get("database", "streamlit"),
        "schema": variables.get("schema", "public"),
        "role": variables.get("role", "accountadmin"),
        "preview_database": variables.get(
            "preview_database", variables.get("database", "streamlit")
        ),
        "preview_schema": variables.get(
            "preview_schema", variables.get("schema", "public")
        ),
        "preview_role": variables.get(
            "preview_role", variables.get("role", "accountadmin")
        ),
        "app_type": variables.get("app_type", "warehouse"),
    }

    # Update variables with required vars
    variables.update(required_vars)

    cmd = [
        "init",
        str(output_path),
        "--template-source",
        str(template_dir),
        "--template",
        template_name,
        "--no-interactive",
    ]

    # Add all variables as -D parameters
    for key, value in variables.items():
        cmd.extend(["-D", f"{key}={value}"])

    if verbose:
        cmd.append("--verbose")
        typer.echo(f"Command: {' '.join(cmd)}")

    if debug:
        cmd.append("--debug")

    return run_snow_command(cmd, capture_output=not verbose)


def get_snow_info() -> list[dict[str, Any]]:
    """Get information about the snow CLI configuration."""
    result = run_snow_command(["--info"])
    return json.loads(result.stdout)



import streamlit as st

# Configure page and navigation
st.set_page_config(
    page_title="Snowflake App Dashboard",
    page_icon="❄️",
    layout="wide",
)

# Define pages with material icons
pages = {
    "Monitoring": [
        st.Page("app_pages/home.py", icon=":material/home:", title="Dashboard"),
        st.Page(
            "app_pages/usage_metrics.py",
            icon=":material/query_stats:",
            title="Usage Metrics",
        ),
        st.Page(
            "app_pages/cost_analysis.py",
            icon=":material/payments:",
            title="Cost Analysis",
        ),
    ]
}

# Set up navigation
current_page = st.navigation(pages)

# Display page title with icon
st.title(f"{current_page.icon} {current_page.title}")

# Run the current page
current_page.run()



from datetime import timedelta

import streamlit as st
from connection import get_table, run_snowpark
from snowflake.snowpark.functions import (
    avg,
    col,
    count,
    current_timestamp,
    date_trunc,
    dateadd,
    lit,
    sum,
)
from streamlit_extras.chart_container import chart_container

group_by = st.radio(
    "Group by",
    ["minute", "hour", "day"],
    index=1,
    horizontal=True,
)

query_history = (
    get_table("snowflake.account_usage.query_history")
    .select(
        date_trunc(group_by, col("start_time")).alias(group_by),
        "execution_time",
        "bytes_scanned",
    )
    .where(col("start_time") >= dateadd("hour", lit(-24), current_timestamp()))
    .limit(100_000)
)

df = query_history.group_by(group_by).agg(
    count(col("execution_time")).alias("query_count"),
    avg(col("execution_time") / 1000).alias("avg_execution_time_s"),
    sum(col("bytes_scanned") / pow(1024, 3)).alias("gb_scanned"),
)

# Query data with 30-minute cache for weekly metrics
pd_df = run_snowpark(df, ttl=timedelta(minutes=30))

st.write("Track your Snowflake query performance and resource utilization over time.")

# Display metrics using chart container
with chart_container(pd_df):
    col1, col2 = st.columns(2)

    with col1:
        st.line_chart(pd_df, x=group_by, y="query_count", use_container_width=True)
        st.caption("Daily Query Count")

    with col2:
        st.line_chart(
            pd_df, x=group_by, y="avg_execution_time_s", use_container_width=True
        )
        st.caption("Average Execution Time (seconds)")

    st.area_chart(pd_df, x=group_by, y="gb_scanned", use_container_width=True)
    st.caption("Data Scanned (GB)")



import json
import subprocess
from pathlib import Path
from typing import Any, Generator
from unittest.mock import MagicMock, patch

import pytest
import toml
import yaml
from typer.testing import CliRunner


@pytest.fixture
def runner() -> CliRunner:
    """Create a CLI runner for testing."""
    return CliRunner()


@pytest.fixture
def mock_snow_cli(
    monkeypatch,
    mock_config_toml: Path,
    mock_get_connection: None,  # noqa: ARG001
) -> list[list[str]]:
    """Mock Snowflake CLI main function."""

    from sisx.utils.snowcli import run_snow_command as real_run_snow_command

    call_log = []

    def fake_run_snow_command(
        cmd_args: list[str],
        capture_output: bool = True,  # noqa: ARG001
        check: bool = True,  # noqa: ARG001
        debug: bool = False,  # noqa: ARG001
    ) -> str:
        call_log.append(cmd_args)
        if "--info" in cmd_args:
            return json.dumps(
                [
                    {
                        "key": "default_config_file_path",
                        "value": str(mock_config_toml),
                    }
                ]
            )
        if "init" in cmd_args:
            return real_run_snow_command(cmd_args)
        return ""

    monkeypatch.setattr("sisx.utils.snowcli.run_snow_command", fake_run_snow_command)

    return call_log


@pytest.fixture
def mock_get_connection(monkeypatch, mock_config_toml: Path):
    """Mock get_connection_by_name function."""

    def transform_connection_info(name: str) -> dict[str, Any]:
        # The connection list command returns this structure, not just the raw toml
        # info.
        raw_info = toml.loads(mock_config_toml.read_text())["connections"][name]

        transformed_info = {
            "connection_name": name,
            "parameters": raw_info,
            "is_default": False,
        }
        if name == "default":
            transformed_info["is_default"] = True
        return transformed_info

    def fake_get_connection_by_name(name: str) -> dict[str, Any]:
        return transform_connection_info(name)

    def fake_get_default_connection() -> dict[str, Any]:
        return transform_connection_info("default")

    monkeypatch.setattr("sisx.cli.get_connection_by_name", fake_get_connection_by_name)
    monkeypatch.setattr("sisx.cli.get_default_connection", fake_get_default_connection)


@pytest.fixture
def mock_config_file(tmp_path: Path) -> Path:
    """Create a mock snowflake.yml file.

    Args:
        tmp_path: Pytest fixture that provides a temporary directory unique to each test function.

    Returns:
        Path: Path to the created mock config file.
    """
    config = {
        "definition_version": "2",
        "env": {
            "name": "test_app",
            "database": "TEST_DB",
            "schema": "PUBLIC",
            "role": "ACCOUNTADMIN",
            "preview_database": "PREVIEW_DB",
            "preview_schema": "PREVIEW_SCHEMA",
            "preview_role": "PREVIEW_ROLE",
            "stage": "CUSTOM_STAGE",
            "app_type": "warehouse",
            "query_warehouse": "COMPUTE_WH",
            "app_warehouse": "STREAMLIT_WH",
            "compute_pool": "COMPUTE_POOL",
        },
        "entities": {
            "streamlit_app": {
                "type": "streamlit",
                "identifier": {
                    "name": "test_app",
                    "schema": "PUBLIC",
                    "database": "TEST_DB",
                },
                "stage": "CUSTOM_STAGE",
                "query_warehouse": "COMPUTE_WH",
                "main_file": "streamlit_app.py",
                "pages_dir": "pages/",
                "artifacts": ["**/*.py", "*.yml", "requirements.txt"],
            }
        },
    }
    config_path = tmp_path / "snowflake.yml"
    with open(config_path, "w") as f:
        yaml.dump(config, f)
    return config_path


@pytest.fixture
def mock_config_toml(tmp_path: Path) -> Path:
    """Create a mock config.toml file.

    Args:
        tmp_path: Pytest fixture that provides a temporary directory unique to each test function.

    Returns:
        Path: Path to the created mock config.toml file.
    """
    config = {
        "connections": {
            "default": {
                "connection_name": "default",
                "account": "test_account",
                "user": "test_user",
                "role": "ACCOUNTADMIN",
                "database": "TEST_DB",
                "schema": "PUBLIC",
                "warehouse": "COMPUTE_WH",
            },
            "test_conn": {
                "connection_name": "test_conn",
                "account": "test_account",
                "user": "test_user",
                "role": "ACCOUNTADMIN",
                "database": "TEST_DB",
                "schema": "PUBLIC",
                "warehouse": "COMPUTE_WH",
            },
        },
    }
    config_path = tmp_path / ".snowflake" / "config.toml"
    config_path.parent.mkdir(parents=True, exist_ok=True)

    with open(config_path, "w") as f:
        toml.dump(config, f)
    return config_path


@pytest.fixture
def mock_snow_info(mock_config_toml: Path) -> Generator[MagicMock, None, None]:
    """Mock snow --info command, but let other subprocess calls run normally."""
    real_subprocess_run = subprocess.run  # Store the original function

    def mock_run_impl(cmd_args, *args, **kwargs):
        # Only mock snow --info command
        if isinstance(cmd_args, list) and "snow" in cmd_args and "--info" in cmd_args:
            mock_result = MagicMock()
            mock_result.stdout = json.dumps(
                [
                    {
                        "key": "default_config_file_path",
                        "value": str(mock_config_toml),
                    }
                ]
            )
            mock_result.returncode = 0
            return mock_result
        # Use the original subprocess.run for all other calls
        return real_subprocess_run(cmd_args, *args, **kwargs)

    with patch("subprocess.run", side_effect=mock_run_impl) as mock_run:
        yield mock_run



from sisx.utils.secrets import _get_connection_settings, create_secrets_file


def test_create_secrets_file_warehouse(tmp_path):
    """Test creating secrets file with warehouse configuration."""
    connection_config = {
        "parameters": {
            "account": "test_account",
            "user": "test_user",
            "password": "test_password",
            "role": "test_role",
            "database": "test_db",
            "schema": "test_schema",
            "warehouse": "test_warehouse",
        }
    }
    compute_config = {"type": "warehouse", "app_warehouse": "test_app_warehouse"}

    create_secrets_file(tmp_path, connection_config, compute_config)

    secrets_path = tmp_path / ".streamlit" / "secrets.toml"
    assert secrets_path.exists()

    content = secrets_path.read_text()
    assert "[connections.snowflake]" in content
    assert 'account = "test_account"' in content
    assert 'user = "test_user"' in content
    assert 'password = "test_password"' in content
    assert 'role = "test_role"' in content
    assert 'database = "test_db"' in content
    assert 'schema = "test_schema"' in content
    assert 'warehouse = "test_app_warehouse"' in content


def test_create_secrets_file_compute_pool(tmp_path):
    """Test creating secrets file with compute pool configuration."""
    connection_config = {
        "parameters": {
            "account": "test_account",
            "user": "test_user",
            "authenticator": "externalbrowser",
            "database": "test_db",
            "schema": "test_schema",
        }
    }
    compute_config = {"type": "compute_pool", "compute_pool": "test_compute_pool"}

    create_secrets_file(tmp_path, connection_config, compute_config)

    secrets_path = tmp_path / ".streamlit" / "secrets.toml"
    assert secrets_path.exists()

    content = secrets_path.read_text()
    assert "[connections.snowflake]" in content
    assert 'account = "test_account"' in content
    assert 'user = "test_user"' in content
    assert 'authenticator = "externalbrowser"' in content
    assert 'database = "test_db"' in content
    assert 'schema = "test_schema"' in content
    assert 'compute_pool = "test_compute_pool"' in content
    assert "warehouse" not in content


def test_get_connection_settings_warehouse():
    """Test _get_connection_settings with warehouse configuration."""
    connection_config = {
        "account": "test_account",
        "user": "test_user",
        "password": "test_password",
        "role": "test_role",
        "database": "test_db",
        "schema": "test_schema",
        "warehouse": "test_warehouse",
        "ignored_key": "should_not_appear",
    }
    compute_config = {"type": "warehouse", "app_warehouse": "test_app_warehouse"}

    settings = _get_connection_settings(connection_config, compute_config)

    assert settings["account"] == "test_account"
    assert settings["user"] == "test_user"
    assert settings["password"] == "test_password"
    assert settings["role"] == "test_role"
    assert settings["database"] == "test_db"
    assert settings["schema"] == "test_schema"
    assert settings["warehouse"] == "test_app_warehouse"
    assert "ignored_key" not in settings


def test_get_connection_settings_compute_pool():
    """Test _get_connection_settings with compute pool configuration."""
    connection_config = {
        "account": "test_account",
        "user": "test_user",
        "authenticator": "externalbrowser",
        "database": "test_db",
        "schema": "test_schema",
        "warehouse": "should_not_appear",
    }
    compute_config = {"type": "compute_pool", "compute_pool": "test_compute_pool"}

    settings = _get_connection_settings(connection_config, compute_config)

    assert settings["account"] == "test_account"
    assert settings["user"] == "test_user"
    assert settings["authenticator"] == "externalbrowser"
    assert settings["database"] == "test_db"
    assert settings["schema"] == "test_schema"
    assert settings["compute_pool"] == "test_compute_pool"
    assert "warehouse" not in settings


def test_get_connection_settings_skip_invalid():
    """Test _get_connection_settings skips invalid values."""
    connection_config = {
        "account": "test_account",
        "password": "****",  # Should be skipped
        "role": None,  # Should be skipped
        "database": 123,  # Should be skipped (not a string)
    }
    compute_config = {"type": "warehouse", "app_warehouse": "test_app_warehouse"}

    settings = _get_connection_settings(connection_config, compute_config)

    assert settings["account"] == "test_account"
    assert settings["warehouse"] == "test_app_warehouse"
    assert "password" not in settings
    assert "role" not in settings
    assert "database" not in settings



from __future__ import annotations

import shutil
import subprocess
from importlib.metadata import PackageNotFoundError
from pathlib import Path
from typing import Annotated, Any

import typer
import yaml

from .utils.secrets import create_secrets_file
from .utils.snowcli import (
    deploy_streamlit,
    drop_spcs_services,
    drop_streamlit,
    init_app,
    list_streamlit_apps,
)
from .utils.templates import (
    get_template_dir,
    get_template_variables,
)
from .utils.toml_config import (
    get_connection_by_name,
    get_default_connection,
)
from .utils.yaml_config import _read_snowflake_config

app = typer.Typer(help="sisx CLI tool for Streamlit in Snowflake applications")

# Type aliases for better readability
ComputeConfig = dict[str, Any]


def _version_callback(value: bool) -> None:
    """Print the version of sisx and exit."""
    if value:
        from importlib.metadata import version

        try:
            v = version("sisx")
            typer.echo(f"sisx version: {v}")
        except PackageNotFoundError:
            typer.echo("sisx version: unknown")
        raise typer.Exit()


@app.callback()
def main_callback(
    version: bool = typer.Option(
        False,
        "--version",
        help="Show the version and exit.",
        callback=_version_callback,
    ),
) -> None:
    """
    sisx CLI tool for Streamlit in Snowflake applications.
    """


@app.command(
    context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def deploy(
    ctx: typer.Context,
    verbose: bool = typer.Option(
        False, "--verbose", "-v", help="Enable verbose output"
    ),
    replace: bool = typer.Option(
        False, "--replace", "-r", help="Replace the Streamlit app if it already exists"
    ),
    open_browser: Annotated[
        bool,
        typer.Option(
            "--no-open", help="Don't open the deployed Streamlit app in a browser"
        ),
    ] = False,
    connection: str | None = typer.Option(
        None, "--connection", "-c", help="Connection to use for deployment"
    ),
    query_warehouse: str | None = typer.Option(
        None, "--query-warehouse", help="Override the query warehouse to use"
    ),
    app_warehouse: str | None = typer.Option(
        None, "--app-warehouse", help="Override the app warehouse to use"
    ),
    debug: bool = typer.Option(False, "--debug", help="Enable debug output"),
) -> None:
    """
    Deploy a Streamlit application using the Snowflake CLI.

    This command wraps the `snow streamlit deploy` command, providing a convenient
    interface for deploying Streamlit applications to Snowflake.

    By default, opens the app in your browser. Use --no-open to disable.
    """
    args = ctx.args
    config = _read_snowflake_config()

    # Get configuration from env section
    env = config.get("env", {})
    name = env.get("name")
    database = env.get("database")
    schema = env.get("schema")
    role = env.get("role")

    if not all([name, database, schema, role]):
        typer.echo("Missing required configuration in snowflake.yml")
        raise typer.Exit(code=1)

    connection_info = _get_connection(connection, verbose, debug)
    connection_name = connection_info["connection_name"]

    try:
        deploy_streamlit(
            name=name,
            database=database,
            schema=schema,
            role=role,
            replace=replace,
            open_browser=not open_browser,
            connection=connection_name,
            query_warehouse=query_warehouse,
            app_warehouse=app_warehouse,
            verbose=verbose,
            debug=debug,
            extra_args=args,
        )
    except Exception as e:
        typer.echo(f"Error deploying app: {e}")
        raise typer.Exit(code=1) from e

    typer.echo(f"App '{name}' successfully deployed!")


@app.command(name="deploy-preview")
def deploy_preview(
    preview_name: str | None = typer.Option(
        None, "--preview-name", help="Name of the preview app"
    ),
    verbose: bool = typer.Option(
        False, "--verbose", "-v", help="Enable verbose output"
    ),
    replace: bool = typer.Option(
        False, "--replace", "-r", help="Replace the Streamlit app if it already exists"
    ),
    open_browser: Annotated[
        bool,
        typer.Option(
            "--no-open", help="Don't open the deployed Streamlit app in a browser"
        ),
    ] = False,
    connection: str | None = typer.Option(
        None, "--connection", "-c", help="Connection to use for deployment"
    ),
    debug: bool = typer.Option(False, "--debug", help="Enable debug output"),
) -> None:
    """
    Deploy a preview version of a Streamlit application.

    This command adds 'preview_' to the beginning of the app name and also adds
    'PREVIEW: ' to the app title, making it easy to identify preview deployments.

    By default, opens the app in your browser. Use --no-open to disable.
    """
    config = _read_snowflake_config()

    # Get configuration from env section
    env = config.get("env", {})
    original_name = env.get("name")
    original_title = env.get("title", "Streamlit App")
    preview_database = env.get("preview_database", env.get("database"))
    preview_schema = env.get("preview_schema", env.get("schema"))
    preview_role = env.get("preview_role", env.get("role"))

    if not all([original_name, preview_database, preview_schema, preview_role]):
        typer.echo("Missing required configuration in snowflake.yml")
        raise typer.Exit(code=1)

    if preview_name is None:
        preview_name = f"preview_{original_name}"

    preview_title = f"PREVIEW: {original_title}"

    if verbose:
        typer.echo(f"Creating preview deployment with name: {preview_name}")
        typer.echo(f"Preview title will be: {preview_title}")
        typer.echo(f"Using database: {preview_database}")
        typer.echo(f"Using schema: {preview_schema}")
        typer.echo(f"Using role: {preview_role}")

    try:
        deploy_streamlit(
            name=preview_name,
            database=preview_database,
            schema=preview_schema,
            role=preview_role,
            title=preview_title,
            replace=replace,
            open_browser=not open_browser,
            connection=connection,
            verbose=verbose,
            debug=debug,
        )
    except Exception as e:
        typer.echo(f"Error deploying app: {e}")
        raise typer.Exit(code=1) from e

    typer.echo(f"App '{preview_name}' successfully deployed!")


def _drop_app(
    app_name: str,
    database: str,
    schema: str,
    connection: str | None = None,
    verbose: bool = False,
    drop_spcs: bool = False,
) -> bool:
    """Helper function to drop a Streamlit app and optionally its SPCS services."""
    # First drop any SPCS services associated with the app if requested
    if drop_spcs:
        _drop_app_spcs_services(app_name, connection, verbose)

    try:
        drop_streamlit(
            app_name=app_name,
            database=database,
            schema=schema,
            connection=connection,
            verbose=verbose,
        )
        full_app_name = f"{database}.{schema}.{app_name}"
        typer.echo(f"Successfully dropped app '{full_app_name}'")
        return True
    except Exception as e:
        typer.echo(f"Error dropping Streamlit app: {e}")
        return False


def _drop_app_spcs_services(
    app_name: str,
    connection: str | None = None,
    verbose: bool = False,
) -> bool:
    """Helper function to drop SPCS services associated with a Streamlit app."""
    if verbose:
        typer.echo(f"Dropping associated SPCS services for {app_name}")

    try:
        drop_spcs_services(
            app_name=app_name,
            connection=connection,
            verbose=verbose,
        )
        return True
    except Exception as e:
        typer.echo(
            f"Warning: Error dropping SPCS services for {app_name} (this is usually ok): {e}"
        )
        return False


@app.command()
def drop(
    verbose: bool = typer.Option(
        False, "--verbose", "-v", help="Enable verbose output"
    ),
    connection: str | None = typer.Option(
        None, "--connection", "-c", help="Connection to use for deployment"
    ),
    force: bool = typer.Option(
        False, "--force", "-f", help="Force drop without confirmation"
    ),
    drop_spcs: bool = typer.Option(
        False, "--drop-spcs", help="Also drop any associated SPCS services"
    ),
) -> None:
    """
    Drop a Streamlit application from Snowflake.

    This command will remove the Streamlit app. If --drop-spcs is specified,
    it will also clean up any associated SPCS services.
    """
    # Read the app name from the yaml file
    try:
        config = yaml.safe_load(Path("snowflake.yml").read_text())
    except Exception as e:
        typer.echo(f"Error reading app configuration from snowflake.yml: {e}")
        raise typer.Exit(code=1) from e

    # Get the app name from env section
    app_name = config.get("env", {}).get("name")
    database = config.get("streamlit", {}).get("database")
    schema = config.get("streamlit", {}).get("schema")

    if verbose:
        typer.echo(
            f"Found configuration: app={app_name}, db={database}, schema={schema}"
        )

    # Create the fully qualified name
    full_app_name = f"{database}.{schema}.{app_name}"

    if verbose:
        typer.echo(f"Dropping Streamlit app: {full_app_name}")

    # Confirm unless --force is specified
    if not force:
        confirm = typer.confirm(
            f"Are you sure you want to drop the app '{full_app_name}'?"
        )
        if not confirm:
            typer.echo("Operation cancelled.")
            raise typer.Exit()

    # Use the helper function to drop the app
    success = _drop_app(app_name, database, schema, connection, verbose, drop_spcs)
    if not success:
        raise typer.Exit(code=1)


@app.command(name="drop-preview")
def drop_preview(
    verbose: bool = typer.Option(
        False, "--verbose", "-v", help="Enable verbose output"
    ),
    connection: str = typer.Option(
        None, "--connection", "-c", help="Connection to use for deployment"
    ),
    force: bool = typer.Option(
        False, "--force", "-f", help="Force drop without confirmation"
    ),
    drop_spcs: bool = typer.Option(
        False, "--drop-spcs", help="Also drop any associated SPCS services"
    ),
):
    """
    Drop a preview version of a Streamlit application from Snowflake.

    This command will remove the preview Streamlit app (with prefix 'preview_').
    If --drop-spcs is specified, it will also clean up any associated SPCS services.
    """
    try:
        config = yaml.safe_load(Path("snowflake.yml").read_text())
    except (yaml.YAMLError, OSError) as e:
        typer.echo(f"Error reading app configuration from snowflake.yml: {e}")
        raise typer.Exit(code=1) from e

    # Get the app name from env section and add preview prefix
    original_name = config.get("env", {}).get("name")
    preview_name = f"preview_{original_name}"
    database = config.get("streamlit", {}).get("database")
    schema = config.get("streamlit", {}).get("schema")

    if verbose:
        typer.echo(
            f"Found configuration: app={preview_name}, db={database}, schema={schema}"
        )

    if not original_name or not database or not schema:
        typer.echo("Error: Missing required configuration in snowflake.yml")
        raise typer.Exit(code=1)

    # Create the fully qualified name
    full_preview_name = f"{database}.{schema}.{preview_name}"

    if verbose:
        typer.echo(f"Dropping preview Streamlit app: {full_preview_name}")

    # Confirm unless --force is specified
    if not force:
        confirm = typer.confirm(
            f"Are you sure you want to drop the preview app '{full_preview_name}'?"
        )
        if not confirm:
            typer.echo("Operation cancelled.")
            raise typer.Exit()

    # Use the helper function to drop the app
    success = _drop_app(preview_name, database, schema, connection, verbose, drop_spcs)
    if not success:
        raise typer.Exit(code=1)


@app.command()
def new(
    output_dir: str | None = typer.Argument(
        None, help="Directory where the new app will be created"
    ),
    force: bool = typer.Option(
        False,
        "--force",
        "-f",
        help="Force creation of app, overwriting existing directory",
    ),
    connection: str | None = typer.Option(
        None,
        "--connection",
        "-c",
        help="Name of the connection to use",
    ),
    verbose: bool = typer.Option(
        False, "--verbose", "-v", help="Enable verbose output"
    ),
    compute_type: str | None = typer.Option(
        "warehouse",
        "--compute-type",
        "-t",
        help="Type of compute to use: 'warehouse' or 'compute_pool'",
    ),
    query_warehouse: str | None = typer.Option(
        None,
        "--query-warehouse",
        "-q",
        help="Warehouse to use for queries (when using warehouse compute type)",
    ),
    app_warehouse: str | None = typer.Option(
        None,
        "--app-warehouse",
        "-a",
        help="Warehouse to use for running the app (when using warehouse compute type)",
    ),
    compute_pool: str | None = typer.Option(
        None,
        "--compute-pool",
        help="Compute pool to use (when using compute_pool compute type)",
    ),
    stage: str | None = typer.Option(
        None, "--stage", help="Stage to use for app artifacts"
    ),
    database: str | None = typer.Option(
        None, "--database", help="Database to use for the app"
    ),
    schema: str | None = typer.Option(
        None, "--schema", help="Schema to use for the app"
    ),
    role: str | None = typer.Option(None, "--role", help="Role to use for the app"),
    debug: bool = typer.Option(False, "--debug", help="Enable debug output"),
) -> None:
    """
    Create a new Streamlit in Snowflake application using a template.

    This command uses snow init to generate a new app from the appropriate template
    based on your compute choice (warehouse or compute pool).

    All configuration can be provided via command line flags for automation, or
    will be prompted for interactively if not provided.
    """
    if verbose:
        typer.echo(f"Creating new Streamlit app in {output_dir}")

    if output_dir is None:
        output_dir = _get_output_dir()

    # Get connection info - either from flag or interactive
    current_connection = _get_connection(connection, verbose, debug)

    # Get compute configuration - either from flags or interactive, defaulting to
    # values from the current connection
    compute_config = {}
    if compute_type is not None and compute_type not in ["warehouse", "compute_pool"]:
        typer.echo("compute-type must be either 'warehouse' or 'compute_pool'")
        raise typer.Exit(code=1)
    if compute_type is None:
        compute_type = "warehouse"
    compute_config["type"] = compute_type

    if query_warehouse is not None:
        compute_config["query_warehouse"] = query_warehouse
    if app_warehouse is not None:
        compute_config["app_warehouse"] = app_warehouse
    if compute_pool is not None:
        compute_config["compute_pool"] = compute_pool
    if stage is not None:
        compute_config["stage"] = stage
    if database is not None:
        compute_config["database"] = database
    if schema is not None:
        compute_config["schema"] = schema
    if role is not None:
        compute_config["role"] = role

    compute_config = _get_compute_configuration(compute_config, current_connection)

    template_name = "app"  # Use the unified template
    compute_config["app_type"] = compute_type  # Add app_type to compute_config

    template_dir = get_template_dir()
    output_path = Path(output_dir)

    if output_path.exists():
        if force:
            shutil.rmtree(output_path)
        else:
            typer.echo(f"Error: Output directory {output_path} already exists")
            raise typer.Exit(code=1)

    template_variables = get_template_variables(output_path, compute_config)

    try:
        init_app(
            output_path=output_path,
            template_dir=template_dir,
            template_name=template_name,
            variables=template_variables,
            verbose=verbose,
            debug=debug,
            force=force,
        )
    except Exception as e:
        typer.echo(f"Error creating new Streamlit app: {e}")
        raise typer.Exit(code=1) from e

    # Delete the unused dependency file based on compute type
    try:
        if compute_type == "compute_pool":
            # For compute pool, delete environment.yml
            env_file = output_path / "environment.yml"
            if env_file.exists():
                env_file.unlink()
                if verbose:
                    typer.echo(
                        "Removed environment.yml as compute_pool app uses requirements.txt"
                    )
        else:
            # For warehouse, delete requirements.txt
            req_file = output_path / "requirements.txt"
            if req_file.exists():
                req_file.unlink()
                if verbose:
                    typer.echo(
                        "Removed requirements.txt as warehouse app uses environment.yml"
                    )
    except OSError as e:
        typer.echo(f"Warning: Error cleaning up unused dependency file: {e}")

    # Create secrets.toml after snow init succeeds
    try:
        create_secrets_file(output_path, current_connection, compute_config)
    except OSError as e:
        typer.echo(f"Error creating secrets.toml: {e}")
        raise typer.Exit(code=1) from e

    typer.echo(f"Successfully created new Streamlit app in {output_dir}!")


def _get_connection(
    name: str | None = None, verbose: bool = False, debug: bool = False
) -> dict[str, Any]:
    """Look up either the named connection or the default connection, and if not found
    then create a new connection."""
    if name is not None:
        try:
            current_connection = get_connection_by_name(name)
        except ValueError:
            # Run the new connection command
            typer.echo(f"Connection '{name}' not found. Creating new connection...")
            cmd = ["new-connection", "--connection-name", name]
            if verbose:
                cmd.append("--verbose")
            if debug:
                cmd.append("--debug")
            app(cmd, standalone_mode=False)
            current_connection = get_connection_by_name(name)
    else:
        try:
            current_connection = get_default_connection()
        except ValueError:
            typer.echo("No default connection found. Creating new connection...")
            cmd = ["new-connection"]
            if verbose:
                cmd.append("--verbose")
            if debug:
                cmd.append("--debug")
            app(cmd, standalone_mode=False)
            current_connection = get_default_connection()

    return current_connection


def _get_output_dir() -> str:
    """Get the output directory from the user."""
    return typer.prompt("Enter the directory where the new app will be created")


def _get_compute_configuration(
    compute_config: dict[str, Any],
    current_connection: dict[str, Any],
) -> dict[str, Any]:
    """Get compute configuration interactively from the user."""

    # Use values from compute_config if they exist, otherwise prompt
    compute_config["database"] = compute_config.get("database") or typer.prompt(
        "What database should be used?",
        default=current_connection.get("database", "streamlit"),
    )
    compute_config["schema"] = compute_config.get("schema") or typer.prompt(
        "What schema should be used?",
        default=current_connection.get("schema", "public"),
    )
    compute_config["role"] = compute_config.get("role") or typer.prompt(
        "What role should be used?",
        default=current_connection.get("role", "ACCOUNTADMIN"),
    )

    # Set preview values to the same as main values if not provided
    if not all(
        key in compute_config
        for key in ["preview_database", "preview_schema", "preview_role"]
    ):
        compute_config["preview_database"] = compute_config["database"]
        compute_config["preview_schema"] = compute_config["schema"]
        compute_config["preview_role"] = compute_config["role"]

    if "type" not in compute_config:
        compute_type = typer.prompt(
            "Choose your compute type:\n1. Warehouse (traditional Snowflake warehouse)\n2. Compute Pool (serverless compute)",
            type=str,
            default="1",
        )
        if compute_type not in ["1", "2"]:
            typer.echo("Invalid compute type selected")
            raise typer.Exit(code=1)
        compute_config["type"] = "warehouse" if compute_type == "1" else "compute_pool"

    if compute_config["type"] == "warehouse":
        if "query_warehouse" not in compute_config:
            compute_config["query_warehouse"] = typer.prompt(
                "What warehouse should be used for queries?",
                default=current_connection.get("warehouse", "compute_wh"),
            )
        if "app_warehouse" not in compute_config:
            compute_config["app_warehouse"] = typer.prompt(
                "What warehouse should be used to run the app?",
                default="SYSTEM$STREAMLIT_NOTEBOOK_WH",
            )
    else:
        if "compute_pool" not in compute_config:
            compute_config["compute_pool"] = typer.prompt(
                "What compute pool should be used?", default="streamlit_compute_pool"
            )

    # Set stage to None if not provided
    if "stage" not in compute_config:
        compute_config["stage"] = None

    return compute_config


@app.command()
def run(
    app_path: str = typer.Argument(
        "streamlit_app.py", help="Path to the Streamlit app file to run"
    ),
) -> None:
    """
    Run a Streamlit application locally.

    This command wraps the `streamlit run` command, providing a convenient
    interface for running Streamlit applications during development.
    """
    # Run uv sync first to make sure all dependencies are installed
    typer.echo("Installing/updating dependencies...")

    subprocess.run(["uv", "sync"], check=True)

    typer.echo(f"Running Streamlit app: {app_path}")

    # Base command, use uv so that it automatically runs with the right dependencies
    cmd = ["uv", "run", "streamlit", "run", app_path]

    try:
        # Execute the command
        subprocess.run(cmd, check=True)
    except Exception as e:
        typer.echo(f"Error running Streamlit app: {e}")
        raise typer.Exit(code=1) from e


@app.command(
    context_settings={"allow_extra_args": True, "ignore_unknown_options": True}
)
def test(ctx: typer.Context) -> None:
    """
    Run the tests for the CLI.
    """
    # Arbitrary arguments are passed to pytest
    args = ctx.args

    subprocess.run(["pytest", *args])


@app.command()
def cleanup(
    filter_text: str | None = typer.Argument(
        None, help="Text to filter apps by (uses preview_ if not specified)"
    ),
    verbose: bool = typer.Option(
        False, "--verbose", "-v", help="Enable verbose output"
    ),
    connection: str | None = typer.Option(
        None, "--connection", "-c", help="Connection to use for deployment"
    ),
    force: bool = typer.Option(
        False, "--force", "-f", help="Force cleanup without confirmation"
    ),
    drop_spcs: bool = typer.Option(
        False, "--drop-spcs", help="Also drop any associated SPCS services"
    ),
) -> None:
    """
    Drop multiple Streamlit applications containing the specified filter text.

    If no filter is provided, uses the text 'preview_' as the filter.
    This is useful for cleaning up development apps in your account.
    """
    if filter_text is None:
        filter_text = "preview_"
        if verbose:
            typer.echo("No filter provided. Using 'preview_' as filter")

    try:
        apps = list_streamlit_apps(
            filter_text=filter_text,
            connection=connection,
            verbose=verbose,
        )
    except Exception as e:
        typer.echo(f"Error listing apps: {e}")
        raise typer.Exit(code=1) from e

    if not apps:
        typer.echo(f"No apps found matching filter: {filter_text}")
        return

    # Display the apps to be deleted
    typer.echo(f"Found {len(apps)} app(s) matching filter '{filter_text}':")
    for app in apps:
        full_name = f"{app['database_name']}.{app['schema_name']}.{app['name']}"
        typer.echo(f" - {full_name}")

    # Confirm deletion unless --force is specified
    if not force:
        confirm = typer.confirm("Are you sure you want to drop all these apps?")
        if not confirm:
            typer.echo("Operation cancelled.")
            raise typer.Exit()

    # Drop each app
    dropped_count = 0
    for app in apps:
        database = app["database_name"]
        schema = app["schema_name"]
        app_name = app["name"]
        full_name = f"{database}.{schema}.{app_name}"

        typer.echo(f"Dropping app: {full_name}")

        # Use the helper function to drop the app
        success = _drop_app(app_name, database, schema, connection, verbose, drop_spcs)
        if success:
            dropped_count += 1

    typer.echo(
        f"Successfully cleaned up {dropped_count} of {len(apps)} app(s) matching filter '{filter_text}'"
    )


@app.command()
def new_connection(
    connection_name: str = typer.Option(
        None,
        "--connection-name",
        "-n",
        help="Name of the new connection.",
    ),
    account: str = typer.Option(
        None,
        "--account",
        "--accountname",
        "-a",
        help="Account name to use when authenticating with Snowflake.",
    ),
    user: str = typer.Option(
        None,
        "--user",
        "--username",
        "-u",
        help="Username to connect to Snowflake.",
    ),
    password: str = typer.Option(
        None,
        "--password",
        "-p",
        help="Snowflake password.",
    ),
    role: str = typer.Option(
        None,
        "--role",
        "--rolename",
        "-r",
        help="Role to use on Snowflake.",
    ),
    warehouse: str = typer.Option(
        None,
        "--warehouse",
        "-w",
        help="Warehouse to use on Snowflake.",
    ),
    database: str = typer.Option(
        None,
        "--database",
        "--dbname",
        "-d",
        help="Database to use on Snowflake.",
    ),
    schema: str = typer.Option(
        None,
        "--schema",
        "--schemaname",
        "-s",
        help="Schema to use on Snowflake.",
    ),
    host: str = typer.Option(
        None,
        "--host",
        "-h",
        help="Host name the connection attempts to connect to Snowflake.",
    ),
    port: int = typer.Option(
        None,
        "--port",
        "-P",
        help="Port to communicate with on the host.",
    ),
    region: str = typer.Option(
        None,
        "--region",
        "-R",
        help="Region name if not the default Snowflake deployment.",
    ),
    authenticator: str = typer.Option(
        None,
        "--authenticator",
        "-A",
        help="Chosen authenticator, if other than password-based",
    ),
    private_key: str = typer.Option(
        None,
        "--private-key",
        "--private-key-file",
        "--private-key-path",
        "-k",
        help="Path to file containing private key",
    ),
    token_file_path: str = typer.Option(
        None,
        "--token-file-path",
        "-t",
        help="Path to file with an OAuth token that should be used when connecting to Snowflake",
    ),
    default: bool = typer.Option(
        False,
        "--default",
        help="If provided the connection will be configured as default connection.",
    ),
    verbose: bool = typer.Option(
        False,
        "--verbose",
        "-v",
        help="Enable verbose output",
    ),
    debug: bool = typer.Option(
        False,
        "--debug",
        help="Enable debug output",
    ),
) -> None:
    """
    Add a new Snowflake connection configuration.

    This command wraps the `snow connection add` command, providing a convenient
    interface for adding new Snowflake connections.
    """
    cmd = ["connection", "add"]

    # Prompt for required values if not provided
    if connection_name is None:
        connection_name = typer.prompt("Enter connection name", default="default")
    if account is None:
        account = typer.prompt("Enter Snowflake account name")
    if user is None:
        user = typer.prompt("Enter Snowflake username")
    if password is None:
        password = typer.prompt("Enter Snowflake password", hide_input=True)
    if role is None:
        role = typer.prompt("Enter role name", default="ACCOUNTADMIN")
    if warehouse is None:
        warehouse = typer.prompt("Enter warehouse name", default="COMPUTE_WH")
    if database is None:
        database = typer.prompt("Enter database name", default="STREAMLIT")
    if schema is None:
        schema = typer.prompt("Enter schema name", default="PUBLIC")

    # Optional values that we'll only prompt for if user wants to provide them
    if (
        typer.confirm("Do you want to specify a custom host?", default=False)
        and host is None
    ):
        host = typer.prompt("Enter host name")

    if (
        typer.confirm("Do you want to specify a custom port?", default=False)
        and port is None
    ):
        port = typer.prompt("Enter port number", type=int)

    if (
        typer.confirm("Do you want to specify a custom region?", default=False)
        and region is None
    ):
        region = typer.prompt("Enter region name")

    if (
        typer.confirm("Do you want to use a different authenticator?", default=False)
        and authenticator is None
    ):
        authenticator = typer.prompt("Enter authenticator")

    if (
        typer.confirm("Do you want to use private key authentication?", default=False)
        and private_key is None
    ):
        private_key = typer.prompt("Enter path to private key file")

    if (
        typer.confirm("Do you want to use OAuth token authentication?", default=False)
        and token_file_path is None
    ):
        token_file_path = typer.prompt("Enter path to OAuth token file")

    if default is None:
        default = typer.confirm("Set this as the default connection?", default=False)

    # Add all non-None options to the command
    if connection_name:
        cmd.extend(["--connection-name", connection_name])
    if account:
        cmd.extend(["--account", account])
    if user:
        cmd.extend(["--user", user])
    if password:
        cmd.extend(["--password", password])
    if role:
        cmd.extend(["--role", role])
    if warehouse:
        cmd.extend(["--warehouse", warehouse])
    if database:
        cmd.extend(["--database", database])
    if schema:
        cmd.extend(["--schema", schema])
    if host:
        cmd.extend(["--host", host])
    if port:
        cmd.extend(["--port", str(port)])
    if region:
        cmd.extend(["--region", region])
    if authenticator:
        cmd.extend(["--authenticator", authenticator])
    if private_key:
        cmd.extend(["--private-key", private_key])
    if token_file_path:
        cmd.extend(["--token-file-path", token_file_path])
    if default:
        cmd.append("--default")
    if verbose:
        cmd.append("--verbose")
    if debug:
        cmd.append("--debug")

    # Disable interactive mode in the snowcli, since we've already gathered
    # all required values
    cmd.append("--no-interactive")

    try:
        from .utils.snowcli import run_snow_command

        run_snow_command(cmd, capture_output=not verbose)
        typer.echo("Successfully added new Snowflake connection!")
    except Exception as e:
        typer.echo(f"Error adding Snowflake connection: {e}")
        raise typer.Exit(code=1) from e


def main() -> None:
    """
    Main entry point for the CLI.
    """
    return app(prog_name="sisx")


if __name__ == "__main__":
    main()



# ruff: noqa: ARG001

import json
import os
from pathlib import Path
from unittest.mock import patch

import pytest
import yaml
from typer.testing import CliRunner

from sisx.cli import app


@pytest.fixture
def mock_config_file(tmp_path):
    """Create a mock snowflake.yml file."""
    config = {
        "env": {
            "name": "test_app",
            "database": "TEST_DB",
            "schema": "PUBLIC",
            "role": "ACCOUNTADMIN",
        },
        "streamlit": {"database": "TEST_DB", "schema": "PUBLIC"},
    }
    config_path = tmp_path / "snowflake.yml"
    with open(config_path, "w") as f:
        yaml.dump(config, f)
    return config_path


@pytest.fixture
def mock_snow_info():
    """Mock snow --info command."""
    with patch("subprocess.run") as mock_run:
        mock_run.return_value.stdout = json.dumps(
            [
                {
                    "key": "default_config_file_path",
                    "value": "/home/user/.snowflake/config.toml",
                }
            ]
        )
        yield mock_run


def test_drop_basic(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    mock_config_file: Path,
    tmp_path: Path,
) -> None:
    """Test basic drop command."""
    os.chdir(tmp_path)

    result = runner.invoke(app, ["drop", "--force"])

    assert result.exit_code == 0
    assert "Successfully dropped app 'TEST_DB.PUBLIC.test_app'" in result.stdout

    # Verify snow object drop was called with correct args
    call_args = mock_snow_cli[-1]
    assert "object" in call_args
    assert "drop" in call_args
    assert "streamlit" in call_args
    assert "TEST_DB.PUBLIC.test_app" in call_args


def test_drop_with_connection(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    mock_config_file: Path,
    tmp_path: Path,
) -> None:
    """Test drop command with connection override."""
    os.chdir(tmp_path)

    result = runner.invoke(app, ["drop", "--connection", "test_conn", "--force"])

    assert result.exit_code == 0

    call_args = mock_snow_cli[-1]
    assert "--connection" in call_args
    assert "test_conn" in call_args


def test_drop_with_spcs(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    mock_config_file: Path,
    tmp_path: Path,
) -> None:
    """Test drop command with SPCS cleanup."""
    os.chdir(tmp_path)

    result = runner.invoke(app, ["drop", "--drop-spcs", "--force"])

    assert result.exit_code == 0
    assert "Successfully dropped app 'TEST_DB.PUBLIC.test_app'" in result.stdout

    # Should have 2 calls - SPCS cleanup and drop
    assert len(mock_snow_cli) == 2

    # Check SPCS cleanup call
    spcs_args = mock_snow_cli[0]
    assert "sql" in spcs_args
    assert "-q" in spcs_args
    assert "CALL SYSTEM\\$DELETE_RUNNING_STREAMLIT_INSTANCES('test_app')" in spcs_args


def test_drop_without_force(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    mock_config_file: Path,
    tmp_path: Path,
) -> None:
    """Test drop command without force flag (should prompt)."""
    os.chdir(tmp_path)

    # Simulate user answering "n" to prompt
    result = runner.invoke(app, ["drop"], input="n\n")

    assert result.exit_code == 0
    assert "Are you sure" in result.stdout
    assert len(mock_snow_cli) == 0


def test_drop_preview_basic(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    mock_config_file: Path,
    tmp_path: Path,
) -> None:
    """Test basic drop-preview command."""
    os.chdir(tmp_path)

    result = runner.invoke(app, ["drop-preview", "--force"])

    assert result.exit_code == 0

    call_args = mock_snow_cli[-1]
    assert any("preview_test_app" in arg for arg in call_args)


def test_drop_preview_with_spcs(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    mock_config_file: Path,
    tmp_path: Path,
) -> None:
    """Test drop-preview command with SPCS cleanup."""
    os.chdir(tmp_path)

    result = runner.invoke(app, ["drop-preview", "--drop-spcs", "--force"])

    assert result.exit_code == 0
    assert len(mock_snow_cli) == 2

    # Check SPCS cleanup call includes preview name
    spcs_args = mock_snow_cli[0]
    assert any("preview_test_app" in arg for arg in spcs_args)


def test_drop_missing_config(
    runner: CliRunner, mock_snow_cli: list[list[str]], tmp_path: Path
) -> None:
    """Test drop command with missing snowflake.yml."""
    os.chdir(tmp_path)

    result = runner.invoke(app, ["drop", "--force"])

    assert result.exit_code == 1
    assert "Error reading app configuration from snowflake.yml" in result.stdout


def test_drop_invalid_config(
    runner: CliRunner, mock_snow_cli: list[list[str]], tmp_path: Path
) -> None:
    """Test drop command with invalid snowflake.yml."""
    os.chdir(tmp_path)

    # Create invalid YAML file
    Path("snowflake.yml").write_text("invalid: yaml: content:")

    result = runner.invoke(app, ["drop", "--force"])

    assert result.exit_code == 1
    assert "Error reading app configuration from snowflake.yml" in result.stdout


def test_drop_requires_config(runner: CliRunner, tmp_path: Path) -> None:
    """Test drop command fails when no config file exists."""
    os.chdir(tmp_path)
    result = runner.invoke(app, ["drop", "--force"])
    assert result.exit_code == 1
    assert "Error reading app configuration from snowflake.yml" in result.stdout


def test_drop_with_config(
    runner: CliRunner, mock_snow_cli: list[list[str]], tmp_path: Path
) -> None:
    """Test drop command with valid config."""
    os.chdir(tmp_path)

    # Create a minimal snowflake.yml
    config = {
        "env": {"name": "test_app"},
        "streamlit": {"database": "TEST_DB", "schema": "PUBLIC"},
    }
    Path("snowflake.yml").write_text(yaml.dump(config))

    result = runner.invoke(app, ["drop", "--force"])
    assert result.exit_code == 0
    assert "Successfully dropped app 'TEST_DB.PUBLIC.test_app'" in result.stdout

    # Verify snow object drop was called with correct args
    call_args = mock_snow_cli[-1]
    assert any("TEST_DB.PUBLIC.test_app" in arg for arg in call_args)


def test_drop_preview_requires_config(runner: CliRunner, tmp_path: Path) -> None:
    """Test drop-preview command fails when no config file exists."""
    os.chdir(tmp_path)
    result = runner.invoke(app, ["drop-preview", "--force"])
    assert result.exit_code == 1
    assert "Error reading app configuration from snowflake.yml" in result.stdout


def test_drop_preview_with_config(
    runner: CliRunner, mock_snow_cli: list[list[str]], tmp_path: Path
) -> None:
    """Test drop-preview command with valid config."""
    os.chdir(tmp_path)

    # Create a minimal snowflake.yml
    config = {
        "env": {"name": "test_app"},
        "streamlit": {"database": "TEST_DB", "schema": "PUBLIC"},
    }
    Path("snowflake.yml").write_text(yaml.dump(config))

    result = runner.invoke(app, ["drop-preview", "--force"])
    assert result.exit_code == 0
    assert "Successfully dropped app 'TEST_DB.PUBLIC.preview_test_app'" in result.stdout

    # Verify snow object drop was called with correct args
    call_args = mock_snow_cli[-1]
    assert any("TEST_DB.PUBLIC.preview_test_app" in arg for arg in call_args)


def test_drop_prompts_without_force(runner: CliRunner, tmp_path: Path) -> None:
    """Test drop command prompts for confirmation without --force."""
    os.chdir(tmp_path)

    # Create a minimal snowflake.yml
    config = {
        "env": {"name": "test_app"},
        "streamlit": {"database": "TEST_DB", "schema": "PUBLIC"},
    }
    Path("snowflake.yml").write_text(yaml.dump(config))

    # Test with 'n' response
    result = runner.invoke(app, ["drop"], input="n\n")
    assert result.exit_code == 0
    assert "Are you sure" in result.stdout
    assert "Operation cancelled" in result.stdout



from datetime import timedelta

import streamlit as st
from connection import run_sql
from streamlit_extras.chart_container import chart_container

# Query data with 1-hour cache for cost metrics
df = run_sql(
    """
with warehouse_metering_history as (
    select
        to_varchar(start_time::date, 'YYYY-MM-DD') as usage_date,
        warehouse_name,
        credits_used,
    from snowflake.account_usage.warehouse_metering_history
    where start_time >= dateadd('day', -7, current_timestamp())
    limit 10000
)
select
    usage_date,
    warehouse_name,
    sum(credits_used) as credits_used,
from warehouse_metering_history
group by usage_date, warehouse_name
order by usage_date asc, warehouse_name asc;
""",
    ttl=timedelta(hours=1),
)  # Cost data updates less frequently

st.write("Monitor your Snowflake credit usage and costs across warehouses.")

# Pivot the data for better visualization
df_pivot = df.pivot(
    index="usage_date", columns="warehouse_name", values="credits_used"
).fillna(0)

# Display metrics using chart container
with chart_container(df):
    # Show stacked bar chart of credits by warehouse
    st.bar_chart(df_pivot, use_container_width=True)
    st.caption("Daily Credits Used by Warehouse")

    # Show metrics
    total_credits = df["credits_used"].sum()
    total_queries = df["query_count"].sum()

    col1, col2 = st.columns(2)
    with col1:
        st.metric("Total Credits Used", f"{total_credits:.1f}")
    with col2:
        st.metric("Total Queries Run", f"{total_queries:,}")

    # Show detailed table with chart container
    st.write("### Daily Warehouse Usage Details")
    st.dataframe(
        df.pivot_table(
            index="warehouse_name",
            values=["credits_used", "query_count"],
            aggfunc="sum",
        )
        .round(2)
        .sort_values(by="credits_used", ascending=False),
        use_container_width=True,
    )



import json
from importlib import resources
from pathlib import Path
from typing import Any


def get_template_dir() -> Path:
    with resources.path("sisx", "templates") as templates_path:
        return templates_path


def get_template_variables(
    output_dir: Path, compute_config: dict[str, Any]
) -> dict[str, Any]:
    variables = {
        "name": output_dir.name,
        "schema": compute_config["schema"],
        "database": compute_config["database"],
        "role": compute_config["role"],
        "preview_database": compute_config.get(
            "preview_database", compute_config["database"]
        ),
        "preview_schema": compute_config.get(
            "preview_schema", compute_config["schema"]
        ),
        "preview_role": compute_config.get("preview_role", compute_config["role"]),
        "app_type": compute_config["app_type"],
    }

    # Add env section variables
    env_variables = {
        "name": variables["name"],
        "schema": variables["schema"],
        "database": variables["database"],
        "role": variables["role"],
        "preview_database": variables["preview_database"],
        "preview_schema": variables["preview_schema"],
        "preview_role": variables["preview_role"],
        "app_type": variables["app_type"],
    }

    if compute_config["type"] == "warehouse":
        variables["query_warehouse"] = compute_config["query_warehouse"]
        variables["app_warehouse"] = compute_config["app_warehouse"]
        variables["artifacts"] = (
            '"**/*.py", "*.yml"'  # No requirements.txt for warehouse apps
        )
        env_variables["query_warehouse"] = compute_config["query_warehouse"]
        env_variables["app_warehouse"] = compute_config["app_warehouse"]
        variables["compute_pool"] = ""
        env_variables["compute_pool"] = ""
    else:
        variables["compute_pool"] = compute_config["compute_pool"]
        variables["artifacts"] = (
            '"**/*.py", "requirements.txt", "*.yml"'  # Include requirements.txt for compute pool apps
        )
        env_variables["compute_pool"] = compute_config["compute_pool"]
        variables["query_warehouse"] = ""
        env_variables["query_warehouse"] = ""
        variables["app_warehouse"] = ""
        env_variables["app_warehouse"] = ""

    if "stage" in compute_config:
        variables["stage"] = compute_config["stage"]
        env_variables["stage"] = compute_config["stage"]

    variables["env"] = json.dumps(env_variables)

    return variables



from datetime import timedelta

import streamlit as st
from connection import run_sql
from streamlit_extras.chart_container import chart_container

# Query data with 15-minute cache for recent activity
df = run_sql(
    """
with query_history as (
    select
        warehouse_name,
        execution_time,
        bytes_scanned,
    from snowflake.account_usage.query_history
    where start_time >= dateadd('hour', -24, current_timestamp())
    limit 10000
)
select
    warehouse_name,
    count(*) as query_count,
    avg(execution_time) / 1000 as avg_execution_time_s,
    sum(bytes_scanned) / power(1024, 3) as gb_scanned
from query_history
group by warehouse_name
order by query_count desc;
""",
    ttl=timedelta(minutes=15),
)  # More frequent updates for recent activity

st.write(
    "Welcome to your Snowflake monitoring dashboard! Use the navigation to explore different views."
)

# Display overview metrics
st.write("### 24 Hour Activity Overview")
with chart_container(df):
    col1, col2, col3 = st.columns(3)

    with col1:
        st.metric("Total Queries", f"{df['query_count'].sum():,}")

    with col2:
        st.metric("Avg Execution Time", f"{df['avg_execution_time_s'].mean():.1f}s")

    with col3:
        st.metric("Data Scanned", f"{df['gb_scanned'].sum():.1f} GB")

    # Show warehouse activity breakdown
    st.write("### Warehouse Activity")
    st.bar_chart(df, x="warehouse_name", y="query_count", use_container_width=True)

group_by = st.radio(
    "Group by",
    ["minute", "hour", "day"],
    index=1,
    horizontal=True,
)


# Example of longer cache for historical trends
historical_df = run_sql(
    f"""
with query_history as (
    select
        date_trunc('{group_by}', start_time) as {group_by},
        bytes_scanned,
    from snowflake.account_usage.query_history
    where start_time >= dateadd('day', -30, current_timestamp())
    limit 100000
)
select
    {group_by},
    count(*) as query_count,
    sum(bytes_scanned) / power(1024, 4) as tb_scanned
from query_history
group by {group_by}
order by {group_by} asc;
""",
    ttl=timedelta(hours=6),
)  # Longer cache for historical data

st.write("### 30 Day Trends")

st.write("#### Query Volume")
with chart_container(historical_df):
    st.line_chart(historical_df, x=group_by, y="query_count", use_container_width=True)

st.write("#### Data Scanned")
with chart_container(historical_df):
    st.line_chart(historical_df, x=group_by, y="tb_scanned", use_container_width=True)



# ruff: noqa: ARG001

import os
from pathlib import Path

import yaml
from typer.testing import CliRunner

from sisx.cli import app


def test_new_command_warehouse_basic(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    mock_get_connection: None,
    tmp_path: Path,
):
    app_path = tmp_path / "streamlit_app"

    """Test basic warehouse app creation."""
    result = runner.invoke(
        app,
        [
            "new",
            str(app_path),
            "--compute-type",
            "warehouse",
            "--query-warehouse",
            "COMPUTE_WH",
            "--app-warehouse",
            "STREAMLIT_WH",
            "--force",
        ],
    )

    call_args = mock_snow_cli[-1]

    assert result.exit_code == 0
    assert "Successfully created new Streamlit app" in result.stdout

    # Verify snow init was called with correct args
    assert "init" in call_args
    assert "--template" in call_args
    assert "app" in call_args
    assert "compute_pool=" in call_args

    # Check config files were created
    assert (app_path / "snowflake.yml").exists()
    assert (app_path / ".streamlit" / "secrets.toml").exists()

    # Check correct dependency file exists and other was deleted
    assert (app_path / "environment.yml").exists()
    assert not (app_path / "requirements.txt").exists()

    # Verify environment.yml content
    env_content = yaml.safe_load((app_path / "environment.yml").read_text())
    assert env_content["name"] == "streamlit_app"
    assert "snowflake" in env_content["channels"]
    assert any(dep.startswith("python=3.11.9") for dep in env_content["dependencies"])
    assert any(
        dep.startswith("streamlit=1.39.0") for dep in env_content["dependencies"]
    )
    assert any(
        dep.startswith("snowflake-snowpark-python=1.23.0")
        for dep in env_content["dependencies"]
    )


def test_new_command_compute_pool_basic(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    mock_config_file: Path,
    tmp_path: Path,
):
    """Test basic compute pool app creation."""
    result = runner.invoke(
        app,
        [
            "new",
            str(tmp_path),
            "--compute-type",
            "compute_pool",
            "--compute-pool",
            "TEST_POOL",
            "--force",
            "--connection",
            "default",
            "--debug",
            "--database",
            "streamlit",
            "--schema",
            "public",
            "--role",
            "ACCOUNTADMIN",
        ],
    )

    print(result.stdout)
    assert result.exit_code == 0
    assert "Successfully created new Streamlit app" in result.stdout

    # Verify snow init was called with correct args
    call_args = mock_snow_cli[-1]
    assert "app" in call_args
    assert "compute_pool=TEST_POOL" in call_args
    assert "query_warehouse=" in call_args

    # Check config files were created
    assert (tmp_path / "snowflake.yml").exists()
    assert (tmp_path / ".streamlit" / "secrets.toml").exists()

    # Check correct dependency file exists and other was deleted
    assert (tmp_path / "requirements.txt").exists()
    assert not (tmp_path / "environment.yml").exists()

    # Verify requirements.txt content
    req_content = (tmp_path / "requirements.txt").read_text().splitlines()
    assert "streamlit>=1.24.0" in req_content
    assert "snowflake-snowpark-python==1.23.0" in req_content
    assert "streamlit-extras" in req_content
    assert "pandas" in req_content
    assert "numpy" in req_content
    assert "plotly" in req_content


def test_new_command_with_preview_settings(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    tmp_path: Path,
):
    """Test app creation with preview deployment settings."""
    result = runner.invoke(
        app,
        [
            "new",
            str(tmp_path),
            "--compute-type",
            "warehouse",
            "--query-warehouse",
            "COMPUTE_WH",
            "--app-warehouse",
            "STREAMLIT_WH",
            "--force",
        ],
    )

    assert result.exit_code == 0

    # Verify snow init was called with correct args
    mock_snow_cli[-1]

    snowflake_yml = tmp_path / "snowflake.yml"
    config = yaml.safe_load(snowflake_yml.read_text())

    # Check preview settings
    assert config["env"]["preview_database"] == config["env"]["database"]
    assert config["env"]["preview_schema"] == config["env"]["schema"]
    assert config["env"]["preview_role"] == config["env"]["role"]


def test_new_command_with_stage(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    tmp_path: Path,
):
    """Test app creation with custom stage."""
    result = runner.invoke(
        app,
        [
            "new",
            str(tmp_path),
            "--compute-type",
            "warehouse",
            "--query-warehouse",
            "COMPUTE_WH",
            "--app-warehouse",
            "STREAMLIT_WH",
            "--stage",
            "CUSTOM_STAGE",
            "--force",
            "--connection",
            "default",
            "--debug",
            "--database",
            "streamlit",
            "--schema",
            "public",
            "--role",
            "ACCOUNTADMIN",
        ],
    )

    assert result.exit_code == 0
    assert "Successfully created new Streamlit app" in result.stdout

    # Verify snow init was called with correct args
    mock_snow_cli[-1]

    # Verify stage was set in the generated snowflake.yml
    snowflake_yml = tmp_path / "snowflake.yml"
    assert snowflake_yml.exists()
    config = yaml.safe_load(snowflake_yml.read_text())
    assert config["env"]["stage"] == "CUSTOM_STAGE"


def test_new_command_existing_directory(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    tmp_path: Path,
):
    """Test app creation when directory exists without force flag."""
    # Create the directory first
    os.makedirs(tmp_path, exist_ok=True)

    result = runner.invoke(
        app,
        [
            "new",
            str(tmp_path),
            "--compute-type",
            "warehouse",
            "--query-warehouse",
            "COMPUTE_WH",
            "--app-warehouse",
            "STREAMLIT_WH",
        ],
    )

    assert result.exit_code == 1
    assert "already exists" in result.stdout


def test_new_command_invalid_compute_type(
    runner: CliRunner,
    mock_snow_cli: list[list[str]],
    tmp_path: Path,
):
    """Test app creation with invalid compute type."""
    result = runner.invoke(
        app,
        ["new", str(tmp_path), "--compute-type", "invalid", "--force"],
    )

    assert result.exit_code == 1
    assert "compute-type must be either 'warehouse' or 'compute_pool'" in result.stdout



from datetime import timedelta

import pandas as pd
import snowflake.snowpark as sp
import streamlit as st
from snowflake.snowpark import Session


def _get_session() -> Session:
    if "snowpark_session" not in st.session_state:
        connection = st.connection("snowflake")
        st.session_state["snowpark_session"] = connection.session()
    return st.session_state["snowpark_session"]


def run_sql(
    query: str,
    ttl: timedelta | int | None = timedelta(hours=2),
    lowercase_columns: bool = True,
):
    """
    Execute a SQL query and cache the results.

    Args:
        query: The SQL query to execute
        ttl: Time-to-live for the cache. Defaults to 2 hours.
            Set to None to use the default cache invalidation.

    Returns:
        pandas.DataFrame: The query results as a pandas DataFrame
    """

    @st.cache_data(ttl=ttl)
    def _run_sql(query: str) -> pd.DataFrame:
        return _get_session().sql(query).to_pandas()

    df = _run_sql(query)

    if lowercase_columns:
        df.columns = df.columns.str.lower()

    return df


def run_snowpark(
    df: sp.DataFrame,
    ttl: timedelta | int | None = timedelta(hours=2),
    lowercase_columns: bool = True,
) -> pd.DataFrame:
    """Converts a snowpark dataframe to a pandas dataframe and caches the result."""

    @st.cache_data(ttl=ttl)
    def _run_snowpark(
        _df: sp.DataFrame, query: str, lowercase_columns: bool
    ) -> pd.DataFrame:
        _ = query
        df = _df.to_pandas()

        if lowercase_columns:
            df.columns = df.columns.str.lower()

        return df

    query = df._plan.queries[0].sql

    return _run_snowpark(df, query, lowercase_columns=lowercase_columns)


@st.cache_resource
def get_table(table_name: str) -> sp.Table:
    return _get_session().table(table_name)



from typer.testing import CliRunner

from sisx.cli import app


def test_version(runner: CliRunner):
    result = runner.invoke(app, ["--version"])
    assert result.exit_code == 0
    assert "sisx version:" in result.stdout
